@@ -14,18 +14,17 @@
 
 import netaddr
 from neutron_lib import constants
+from neutron_lib.objects import common_types
 from neutron_lib.utils import net as net_utils
 from oslo_log import log as logging
 from oslo_utils import versionutils
 from oslo_versionedobjects import fields as obj_fields
 
-from neutron.common import _constants
 from neutron.db.models import dns as dns_models
 from neutron.db.models import l3
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import models_v2
 from neutron.objects import base
-from neutron.objects import common_types
 from neutron.objects.db import api as obj_db_api
 from neutron.objects.qos import binding
 from neutron.plugins.ml2 import models as ml2_models
@@ -215,6 +214,14 @@ class IPAllocation(base.NeutronDbObject):
         if alloc_db:
             return True
 
+    @classmethod
+    def delete_alloc_by_subnet_id(cls, context, subnet_id):
+        allocs = context.session.query(models_v2.IPAllocation).filter_by(
+            subnet_id=subnet_id).all()
+        for alloc in allocs:
+            alloc_obj = super(IPAllocation, cls)._load_object(context, alloc)
+            alloc_obj.delete()
+
 
 @base.NeutronObjectRegistry.register
 class PortDNS(base.NeutronDbObject):
@@ -416,25 +423,14 @@ class Port(base.NeutronDbObject):
                                             **kwargs)
 
     @classmethod
-    def get_auto_deletable_port_ids_and_proper_port_count_by_segment(
-            cls, context, segment_id):
-
+    def get_port_ids_filter_by_segment_id(cls, context, segment_id):
         query = context.session.query(models_v2.Port.id)
         query = query.join(
             ml2_models.PortBindingLevel,
             ml2_models.PortBindingLevel.port_id == models_v2.Port.id)
         query = query.filter(
             ml2_models.PortBindingLevel.segment_id == segment_id)
-
-        q_delete = query.filter(
-            models_v2.Port.device_owner.in_(
-                _constants.AUTO_DELETE_PORT_OWNERS))
-
-        q_proper = query.filter(
-            ~models_v2.Port.device_owner.in_(
-                _constants.AUTO_DELETE_PORT_OWNERS))
-
-        return ([r.id for r in q_delete.all()], q_proper.count())
+        return [p.id for p in query]
 
     @classmethod
     def modify_fields_to_db(cls, fields):
