@@ -12,7 +12,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import multiprocessing
 import os
+import time
 
 import fixtures
 
@@ -56,3 +58,28 @@ class AdminDirFixture(fixtures.Fixture):
         delete_cmd = ['rm', '-r', self.directory]
         utils.execute(create_cmd, run_as_root=True)
         self.addCleanup(utils.execute, delete_cmd, run_as_root=True)
+
+
+class SleepyProcessFixture(fixtures.Fixture):
+    """Process fixture to perform time.sleep for a given number of seconds."""
+
+    def __init__(self, timeout=60):
+        super(SleepyProcessFixture, self).__init__()
+        self.timeout = timeout
+
+    @staticmethod
+    def yawn(seconds):
+        time.sleep(seconds)
+
+    def _setUp(self):
+        self.process = multiprocessing.Process(target=self.yawn,
+                                               args=[self.timeout])
+        self.process.start()
+        self.addCleanup(self.destroy)
+
+    def destroy(self):
+        self.process.terminate()
+
+    @property
+    def pid(self):
+        return self.process.pid
