@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import re
 import warnings
 
 import mock
@@ -20,15 +21,14 @@ import netaddr
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib.plugins.ml2 import api
 from oslo_utils import uuidutils
 from sqlalchemy.orm import exc
 from sqlalchemy.orm import query
 
-from neutron.db import api as db_api
 from neutron.db import db_base_plugin_v2
 from neutron.db.models import l3 as l3_models
-from neutron.db import models_v2
 from neutron.db import segments_db
 from neutron.objects import network as network_obj
 from neutron.objects import ports as port_obj
@@ -51,7 +51,7 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         network_obj.Network(self.ctx, id=network_id).create()
 
     def _setup_neutron_port(self, network_id, port_id):
-        mac_address = db_base_plugin_v2.NeutronDbPluginV2._generate_mac()
+        mac_address = db_base_plugin_v2.NeutronDbPluginV2._generate_macs()[0]
         port = port_obj.Port(self.ctx,
                              id=port_id,
                              network_id=network_id,
@@ -63,11 +63,13 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         port.create()
         return port
 
-    def _setup_neutron_portbinding(self, port_id, vif_type, host):
-        with db_api.context_manager.writer.using(self.ctx):
+    def _setup_neutron_portbinding(self, port_id, vif_type, host,
+                                   status=constants.ACTIVE):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             self.ctx.session.add(models.PortBinding(port_id=port_id,
                                                     vif_type=vif_type,
-                                                    host=host))
+                                                    host=host,
+                                                    status=status))
 
     @staticmethod
     def _sort_segments(segments):
@@ -218,10 +220,13 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
         network_id = uuidutils.generate_uuid()
         port_id = uuidutils.generate_uuid()
         host = 'fake_host'
+        other_host = 'other_fake_host'
         vif_type = portbindings.VIF_TYPE_UNBOUND
         self._setup_neutron_network(network_id)
         self._setup_neutron_port(network_id, port_id)
         self._setup_neutron_portbinding(port_id, vif_type, host)
+        self._setup_neutron_portbinding(port_id, vif_type, other_host,
+                                        status=constants.INACTIVE)
 
         port_host = ml2_db.get_port_binding_host(self.ctx, port_id)
         self.assertEqual(host, port_host)
@@ -283,6 +288,18 @@ class Ml2DBTestCase(testlib_api.SqlTestCase):
                                                         port['mac_address'])
         self.assertEqual(port_id, observed_port.id)
 
+    def test_generating_multiple_mac_addresses(self):
+        mac_regex = "^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$"
+
+        macs = db_base_plugin_v2.NeutronDbPluginV2._generate_macs()
+        self.assertEqual(1, len(macs))
+        self.assertIsNotNone(re.search(mac_regex, macs[0]))
+
+        macs = db_base_plugin_v2.NeutronDbPluginV2._generate_macs(5)
+        self.assertEqual(5, len(macs))
+        for mac in macs:
+            self.assertIsNotNone(re.search(mac_regex, mac))
+
 
 class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
 
@@ -292,12 +309,12 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
         self.setup_coreplugin(PLUGIN_NAME)
 
     def _setup_neutron_network(self, network_id, port_ids):
-        with db_api.context_manager.writer.using(self.ctx):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             network_obj.Network(self.ctx, id=network_id).create()
             ports = []
             for port_id in port_ids:
                 mac_address = (db_base_plugin_v2.NeutronDbPluginV2.
-                               _generate_mac())
+                               _generate_macs()[0])
                 port = port_obj.Port(self.ctx,
                                      id=port_id,
                                      network_id=network_id,
@@ -318,7 +335,7 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
 
     def _setup_distributed_binding(self, network_id,
                                    port_id, router_id, host_id):
-        with db_api.context_manager.writer.using(self.ctx):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             record = models.DistributedPortBinding(
                 port_id=port_id,
                 host=host_id,
@@ -410,36 +427,38 @@ class Ml2DvrDBTestCase(testlib_api.SqlTestCase):
     def test_distributed_port_binding_deleted_by_port_deletion(self):
         network_id = uuidutils.generate_uuid()
         network_obj.Network(self.ctx, id=network_id).create()
-        with db_api.context_manager.writer.using(self.ctx):
-            device_owner = constants.DEVICE_OWNER_DVR_INTERFACE
-            port = models_v2.Port(
-                id='port_id',
-                network_id=network_id,
-                mac_address='00:11:22:33:44:55',
-                admin_state_up=True,
-                status=constants.PORT_STATUS_ACTIVE,
-                device_id='device_id',
-                device_owner=device_owner)
-            self.ctx.session.add(port)
-            binding_kwarg = {
-                'port_id': 'port_id',
-                'host': 'host',
-                'vif_type': portbindings.VIF_TYPE_UNBOUND,
-                'vnic_type': portbindings.VNIC_NORMAL,
-                'router_id': 'router_id',
-                'status': constants.PORT_STATUS_DOWN
-            }
-            self.ctx.session.add(models.DistributedPortBinding(
-                **binding_kwarg))
-            binding_kwarg['host'] = 'another-host'
-            self.ctx.session.add(models.DistributedPortBinding(
-                **binding_kwarg))
+        device_owner = constants.DEVICE_OWNER_DVR_INTERFACE
+        port = port_obj.Port(
+            self.ctx,
+            id=uuidutils.generate_uuid(),
+            network_id=network_id,
+            mac_address=netaddr.EUI('00-11-22-33-44-55'),
+            admin_state_up=True,
+            status=constants.PORT_STATUS_ACTIVE,
+            device_id='device_id',
+            device_owner=device_owner)
+        port.create()
+        port_obj.DistributedPortBinding(
+            self.ctx,
+            port_id=port.id,
+            host='host',
+            vif_type=portbindings.VIF_TYPE_UNBOUND,
+            vnic_type=portbindings.VNIC_NORMAL,
+            router_id='router_id',
+            status=constants.PORT_STATUS_DOWN).create()
+        port_obj.DistributedPortBinding(
+            self.ctx,
+            port_id=port.id,
+            host='another-host',
+            vif_type=portbindings.VIF_TYPE_UNBOUND,
+            vnic_type=portbindings.VNIC_NORMAL,
+            router_id='router_id',
+            status=constants.PORT_STATUS_DOWN).create()
         with warnings.catch_warnings(record=True) as warning_list:
-            with db_api.context_manager.writer.using(self.ctx):
-                self.ctx.session.delete(port)
+            port.delete()
             self.assertEqual(
                 [], warning_list,
                 'Warnings: %s' % ';'.join([str(w) for w in warning_list]))
         ports = ml2_db.get_distributed_port_bindings(self.ctx,
-                                                     'port_id')
+                                                     port.id)
         self.assertEqual(0, len(ports))
